@@ -7,6 +7,7 @@
 
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
+#include <orc/orconce.h>
 
 /**
  * SECTION:orc
@@ -28,35 +29,39 @@ void _orc_compiler_init(void);
 void
 orc_init (void)
 {
-  static int _inited = 0;
-  if (_inited) return;
+  static volatile int inited = FALSE;
 
-  _inited = 1;
+  if (!inited) {
+    orc_global_mutex_lock ();
+    if (!inited) {
+      ORC_ASSERT(sizeof(OrcExecutor) == sizeof(OrcExecutorAlt));
 
-  ORC_ASSERT(sizeof(OrcExecutor) == sizeof(OrcExecutorAlt));
-
-  _orc_debug_init();
-  _orc_once_init();
-  _orc_compiler_init();
-  orc_opcode_init();
-  orc_c_init();
+      _orc_debug_init();
+      _orc_compiler_init();
+      orc_opcode_init();
+      orc_c_init();
 #ifdef ENABLE_BACKEND_C64X
-  orc_c64x_c_init();
+      orc_c64x_c_init();
 #endif
 #ifdef ENABLE_BACKEND_MMX
-  orc_mmx_init();
+      orc_mmx_init();
 #endif
 #ifdef ENABLE_BACKEND_SSE
-  orc_sse_init();
+      orc_sse_init();
 #endif
 #ifdef ENABLE_BACKEND_ALTIVEC
-  orc_powerpc_init();
+      orc_powerpc_init();
 #endif
 #ifdef ENABLE_BACKEND_ARM
-  orc_arm_init();
+      orc_arm_init();
 #endif
 #ifdef ENABLE_BACKEND_NEON
-  orc_neon_init();
+      orc_neon_init();
 #endif
+
+      inited = TRUE;
+    }
+    orc_global_mutex_unlock ();
+  }
 }
 
