@@ -45,6 +45,7 @@ static char **_orc_compiler_flag_list;
 int _orc_compiler_flag_backup;
 int _orc_compiler_flag_emulate;
 int _orc_compiler_flag_debug;
+int _orc_compiler_flag_randomize;
 
 void
 _orc_compiler_init (void)
@@ -59,6 +60,7 @@ _orc_compiler_init (void)
   _orc_compiler_flag_backup = orc_compiler_flag_check ("backup");
   _orc_compiler_flag_emulate = orc_compiler_flag_check ("emulate");
   _orc_compiler_flag_debug = orc_compiler_flag_check ("debug");
+  _orc_compiler_flag_randomize = orc_compiler_flag_check ("randomize");
 }
 
 int
@@ -89,10 +91,10 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
   }
 
   roff = 0;
-#if 0
-  /* for testing */
-  roff = rand()&0xf;
-#endif
+  if (_orc_compiler_flag_randomize) {
+    /* for testing */
+    roff = rand()&0x1f;
+  }
 
   for(i=0;i<32;i++){
     reg = offset + ((roff + i)&0x1f);
@@ -115,7 +117,7 @@ orc_compiler_allocate_register (OrcCompiler *compiler, int data_reg)
   }
 
   if (data_reg || !compiler->allow_gp_on_stack) {
-    ORC_COMPILER_ERROR (compiler, "register overflow for %s reg",
+    orc_compiler_error (compiler, "register overflow for %s reg",
         data_reg ? "vector" : "gp");
     compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
@@ -308,7 +310,7 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   }
 
   if (program->backup_func && _orc_compiler_flag_backup) {
-    ORC_COMPILER_ERROR(compiler, "Compilation disabled");
+    orc_compiler_error (compiler, "Compilation disabled, using backup");
     compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
     goto error;
   }
@@ -316,6 +318,7 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   if (_orc_compiler_flag_emulate || target == NULL) {
     program->code_exec = (void *)orc_executor_emulate;
     program->orccode->exec = (void *)orc_executor_emulate;
+    orc_compiler_error (compiler, "Compilation disabled, using emulation");
     compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
     goto error;
   }
@@ -361,15 +364,23 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   }
   free (compiler->code);
   compiler->code = NULL;
+  if (compiler->output_insns) free (compiler->output_insns);
   free (compiler);
   ORC_INFO("finished compiling (success)");
 
   return result;
 error:
 
-  ORC_WARNING("program %s failed to compile, reason %d",
-      program->name, compiler->result);
+  if (compiler->error_msg) {
+    ORC_WARNING ("program %s failed to compile, reason: %s",
+        program->name, compiler->error_msg);
+  } else {
+    ORC_WARNING("program %s failed to compile, reason %d",
+        program->name, compiler->result);
+  }
   result = compiler->result;
+  if (program->error_msg) free (program->error_msg);
+  program->error_msg = compiler->error_msg;
   if (result == 0) {
     result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
@@ -383,6 +394,7 @@ error:
   }
   free (compiler->code);
   compiler->code = NULL;
+  if (compiler->output_insns) free (compiler->output_insns);
   free (compiler);
   ORC_INFO("finished compiling (fail)");
   return result;
@@ -410,7 +422,7 @@ orc_compiler_check_sizes (OrcCompiler *compiler)
       if (opcode->dest_size[j] == 0) continue;
       if (multiplier * opcode->dest_size[j] !=
           compiler->vars[insn->dest_args[j]].size) {
-        ORC_COMPILER_ERROR(compiler, "size mismatch, opcode %s dest[%d] is %d should be %d",
+        ORC_COMPILER_ERROR (compiler, "size mismatch, opcode %s dest[%d] is %d should be %d",
             opcode->name, j, compiler->vars[insn->dest_args[j]].size,
             multiplier * opcode->dest_size[j]);
         compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
@@ -620,8 +632,8 @@ orc_compiler_assign_rules (OrcCompiler *compiler)
         compiler->target_flags);
 
     if (insn->rule == NULL || insn->rule->emit == NULL) {
-      ORC_COMPILER_ERROR(compiler, "No rule for: %s on target %s",
-          insn->opcode->name, compiler->target->name);
+      orc_compiler_error (compiler, "no code generation rule for %s on "
+          "target %s", insn->opcode->name, compiler->target->name);
       compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
       return;
     }
@@ -658,12 +670,8 @@ orc_compiler_get_temp_reg (OrcCompiler *compiler)
 
   ORC_DEBUG("at insn %d %s", compiler->insn_index,
       compiler->insns[compiler->insn_index].opcode->name);
-  for(j=0;j<8;j++){
-    ORC_DEBUG("xmm%d: %d %d", j, compiler->valid_regs[ORC_VEC_REG_BASE + j],
-        compiler->alloc_regs[ORC_VEC_REG_BASE + j]);
-  }
 
-  for(j=compiler->min_temp_reg;j<ORC_VEC_REG_BASE+16;j++){
+  for(j=compiler->min_temp_reg;j<ORC_VEC_REG_BASE+32;j++){
     if (compiler->valid_regs[j] && !compiler->alloc_regs[j]) {
       compiler->min_temp_reg = j+1;
       if (compiler->max_used_temp_reg < j) compiler->max_used_temp_reg = j;
@@ -671,7 +679,7 @@ orc_compiler_get_temp_reg (OrcCompiler *compiler)
     }
   }
 
-  ORC_COMPILER_ERROR(compiler,"no temporary register available");
+  orc_compiler_error (compiler, "no temporary register available");
   compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
 
   return 0;
@@ -825,7 +833,7 @@ orc_compiler_global_reg_alloc (OrcCompiler *compiler)
       case ORC_VAR_TYPE_TEMP:
         break;
       default:
-        ORC_COMPILER_ERROR(compiler, "bad vartype");
+        orc_compiler_error (compiler, "bad vartype");
         compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         break;
     }
@@ -1160,12 +1168,7 @@ orc_compiler_get_constant_reg (OrcCompiler *compiler)
     compiler->alloc_regs[j] = 1;
   }
 
-  for(j=0;j<8;j++){
-    ORC_DEBUG("xmm%d: %d %d", j, compiler->valid_regs[ORC_VEC_REG_BASE + j],
-        compiler->alloc_regs[ORC_VEC_REG_BASE + j]);
-  }
-
-  for(j=compiler->max_used_temp_reg;j<ORC_VEC_REG_BASE+16;j++){
+  for(j=compiler->max_used_temp_reg;j<ORC_VEC_REG_BASE+32;j++){
     if (compiler->valid_regs[j] && !compiler->alloc_regs[j]) {
       return j;
     }
@@ -1174,3 +1177,30 @@ orc_compiler_get_constant_reg (OrcCompiler *compiler)
   return 0;
 }
 
+#define ORC_COMPILER_ERROR_BUFFER_SIZE 200
+
+void
+orc_compiler_error_valist (OrcCompiler *compiler, const char *fmt,
+    va_list args)
+{
+  char *s;
+
+  if (compiler->error_msg) return;
+
+  s = malloc (ORC_COMPILER_ERROR_BUFFER_SIZE);
+  vsprintf (s, fmt, args);
+  compiler->error_msg = s;
+  compiler->error = TRUE;
+  compiler->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
+}
+
+void
+orc_compiler_error (OrcCompiler *compiler, const char *fmt, ...)
+{
+  va_list var_args;
+
+  va_start (var_args, fmt);
+  orc_compiler_error_valist (compiler, fmt, var_args);
+  va_end (var_args);
+}
+
