@@ -8,6 +8,7 @@
 
 
 int error = FALSE;
+int verbose = FALSE;
 
 void test_opcode_src (OrcStaticOpcode *opcode);
 void test_opcode_const (OrcStaticOpcode *opcode);
@@ -29,7 +30,7 @@ main (int argc, char *argv[])
   opcode_set = orc_opcode_set_get ("sys");
 
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s src %d,%d,%d,%d */\n",
+    if (verbose) printf("%s src %d,%d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].dest_size[1],
@@ -38,7 +39,7 @@ main (int argc, char *argv[])
     test_opcode_src (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s const %d,%d,%d */\n",
+    if (verbose) printf("%s const %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -46,7 +47,7 @@ main (int argc, char *argv[])
     test_opcode_const (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s param %d,%d,%d */\n",
+    if (verbose) printf("%s param %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -54,7 +55,7 @@ main (int argc, char *argv[])
     test_opcode_param (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s inplace %d,%d,%d */\n",
+    if (verbose) printf("%s inplace %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -62,7 +63,7 @@ main (int argc, char *argv[])
     test_opcode_inplace (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s src 2d %d,%d,%d */\n",
+    if (verbose) printf("%s src 2d %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -70,7 +71,7 @@ main (int argc, char *argv[])
     test_opcode_src_2d (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s src const n %d,%d,%d */\n",
+    if (verbose) printf("%s src const n %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -78,7 +79,7 @@ main (int argc, char *argv[])
     test_opcode_src_const_n (opcode_set->opcodes + i);
   }
   for(i=0;i<opcode_set->n_opcodes;i++){
-    printf("/* %s src const n 2d %d,%d,%d */\n",
+    if (verbose) printf("%s src const n 2d %d,%d,%d\n",
         opcode_set->opcodes[i].name,
         opcode_set->opcodes[i].dest_size[0],
         opcode_set->opcodes[i].src_size[0],
@@ -146,32 +147,43 @@ test_opcode_const (OrcStaticOpcode *opcode)
   char s[40];
   int ret;
   int flags = 0;
+  int args[4] = { -1, -1, -1, -1 };
+  int n_args = 0;
 
   if (opcode->src_size[1] == 0) {
     return;
   }
-
   p = orc_program_new ();
   if (opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR) {
-    orc_program_add_accumulator (p, opcode->dest_size[0], "d1");
+    args[n_args++] =
+      orc_program_add_accumulator (p, opcode->dest_size[0], "d1");
   } else {
-    orc_program_add_destination (p, opcode->dest_size[0], "d1");
+    args[n_args++] =
+      orc_program_add_destination (p, opcode->dest_size[0], "d1");
   }
   if (opcode->dest_size[1] != 0) {
-    orc_program_add_destination (p, opcode->dest_size[1], "d2");
+    args[n_args++] =
+      orc_program_add_destination (p, opcode->dest_size[1], "d2");
+  }
+  args[n_args++] =
+    orc_program_add_source (p, opcode->src_size[0], "s1");
+  args[n_args++] =
+    orc_program_add_constant (p, opcode->src_size[1], 1, "c1");
+  if (opcode->src_size[2]) {
+    args[n_args++] =
+      orc_program_add_constant (p, opcode->src_size[2], 1, "c2");
   }
-  orc_program_add_source (p, opcode->src_size[0], "s1");
-  orc_program_add_constant (p, opcode->src_size[1], 1, "c1");
 
   if ((opcode->flags & ORC_STATIC_OPCODE_FLOAT_SRC) ||
       (opcode->flags & ORC_STATIC_OPCODE_FLOAT_DEST)) {
     flags = ORC_TEST_FLAGS_FLOAT;
   }
 
-  sprintf(s, "test_c_%s", opcode->name);
+  sprintf(s, "test_const_%s", opcode->name);
   orc_program_set_name (p, s);
 
-  orc_program_append_str (p, opcode->name, "d1", "s1", "c1");
+  orc_program_append_2 (p, opcode->name, 0, args[0], args[1],
+      args[2], args[3]);
 
   ret = orc_test_compare_output_full (p, flags);
   if (!ret) {
@@ -189,31 +201,43 @@ test_opcode_param (OrcStaticOpcode *opcode)
   char s[40];
   int ret;
   int flags = 0;
+  int args[4] = { -1, -1, -1, -1 };
+  int n_args = 0;
 
   if (opcode->src_size[1] == 0) {
     return;
   }
   p = orc_program_new ();
   if (opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR) {
-    orc_program_add_accumulator (p, opcode->dest_size[0], "d1");
+    args[n_args++] =
+      orc_program_add_accumulator (p, opcode->dest_size[0], "d1");
   } else {
-    orc_program_add_destination (p, opcode->dest_size[0], "d1");
+    args[n_args++] =
+      orc_program_add_destination (p, opcode->dest_size[0], "d1");
   }
   if (opcode->dest_size[1] != 0) {
-    orc_program_add_destination (p, opcode->dest_size[1], "d2");
+    args[n_args++] =
+      orc_program_add_destination (p, opcode->dest_size[1], "d2");
+  }
+  args[n_args++] =
+    orc_program_add_source (p, opcode->src_size[0], "s1");
+  args[n_args++] =
+    orc_program_add_parameter (p, opcode->src_size[1], "p1");
+  if (opcode->src_size[2]) {
+    args[n_args++] =
+      orc_program_add_parameter (p, opcode->src_size[2], "p2");
   }
-  orc_program_add_source (p, opcode->src_size[0], "s1");
-  orc_program_add_parameter (p, opcode->src_size[1], "p1");
 
   if ((opcode->flags & ORC_STATIC_OPCODE_FLOAT_SRC) ||
       (opcode->flags & ORC_STATIC_OPCODE_FLOAT_DEST)) {
     flags = ORC_TEST_FLAGS_FLOAT;
   }
 
-  sprintf(s, "test_c_%s", opcode->name);
+  sprintf(s, "test_p_%s", opcode->name);
   orc_program_set_name (p, s);
 
-  orc_program_append_str (p, opcode->name, "d1", "s1", "p1");
+  orc_program_append_2 (p, opcode->name, 0, args[0], args[1],
+      args[2], args[3]);
 
   ret = orc_test_compare_output_full (p, flags);
   if (!ret) {
