@@ -124,6 +124,7 @@ main (int argc, char *argv[])
         target = argv[i+1];
         i++;
       } else {
+        help();
       }
     } else if (strcmp(argv[i], "--inline") == 0) {
       use_inline = TRUE;
@@ -226,6 +227,11 @@ main (int argc, char *argv[])
   n_programs = n;
   printf("%s", log);
 
+  if (programs == NULL) {
+    printf("no programs\n");
+    exit(1);
+  }
+
   if (init_function == NULL) {
     init_function = orc_parse_get_init_function (programs[0]);
   }
@@ -711,14 +717,10 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
     if (use_code) {
       fprintf(output, "  OrcCode *c = _orc_code_%s;\n", p->name);
     } else {
-      if (use_code) {
-        fprintf(output, "  OrcCode *c = _orc_code_%s;\n", p->name);
-      } else {
-        fprintf(output, "  OrcProgram *p = _orc_program_%s;\n", p->name);
-      }
+      fprintf(output, "  OrcProgram *p = _orc_program_%s;\n", p->name);
     }
   } else {
-    fprintf(output, "  static int p_inited = 0;\n");
+    fprintf(output, "  static volatile int p_inited = 0;\n");
     if (use_code) {
       fprintf(output, "  static OrcCode *c = 0;\n");
     } else {
@@ -861,6 +863,21 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     fprintf(output, "      orc_program_set_constant_n (p, %d);\n",
         p->constant_n);
   }
+  if (p->n_multiple != 0) {
+    REQUIRE(0,4,14,1);
+    fprintf(output, "      orc_program_set_n_multiple (p, %d);\n",
+        p->constant_n);
+  }
+  if (p->n_minimum != 0) {
+    REQUIRE(0,4,14,1);
+    fprintf(output, "      orc_program_set_n_minimum (p, %d);\n",
+        p->constant_n);
+  }
+  if (p->n_maximum != 0) {
+    REQUIRE(0,4,14,1);
+    fprintf(output, "      orc_program_set_n_maximum (p, %d);\n",
+        p->constant_n);
+  }
   if (p->is_2d) {
     fprintf(output, "      orc_program_set_2d (p);\n");
     if (p->constant_m != 0) {
@@ -876,15 +893,28 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
   for(i=0;i<4;i++){
     var = &p->vars[ORC_VAR_D1 + i];
     if (var->size) {
-      fprintf(output, "      orc_program_add_destination (p, %d, \"%s\");\n",
-          var->size, varnames[ORC_VAR_D1 + i]);
+      if (var->alignment != var->size) {
+        REQUIRE(0,4,14,1);
+        fprintf(output, "      orc_program_add_destination_full (p, %d, \"%s\", 0, %d);\n",
+            var->size, varnames[ORC_VAR_D1 + i], var->alignment);
+      } else {
+        fprintf(output, "      orc_program_add_destination (p, %d, \"%s\");\n",
+            var->size, varnames[ORC_VAR_D1 + i]);
+      }
     }
   }
   for(i=0;i<8;i++){
     var = &p->vars[ORC_VAR_S1 + i];
     if (var->size) {
-      fprintf(output, "      orc_program_add_source (p, %d, \"%s\");\n",
-          var->size, varnames[ORC_VAR_S1 + i]);
+      if (var->alignment != var->size) {
+        REQUIRE(0,4,14,1);
+        fprintf(output, "      orc_program_add_source_full (p, %d, \"%s\", 0, %d);\n",
+            var->size, varnames[ORC_VAR_S1 + i],
+            var->alignment);
+      } else {
+        fprintf(output, "      orc_program_add_source (p, %d, \"%s\");\n",
+            var->size, varnames[ORC_VAR_S1 + i]);
+      }
     }
   }
   for(i=0;i<4;i++){
@@ -1002,10 +1032,10 @@ output_init_function (FILE *output)
       fprintf(output, "  {\n");
       fprintf(output, "    /* %s */\n", programs[i]->name);
       fprintf(output, "    OrcProgram *p;\n");
-      fprintf(output, "    \n");
+      fprintf(output, "\n");
       output_program_generation (programs[i], output, FALSE);
       fprintf(output, "\n");
-      fprintf(output, "      orc_program_compile (p);\n");
+      fprintf(output, "    orc_program_compile (p);\n");
       fprintf(output, "\n");
       if (use_code) {
         fprintf(output, "    _orc_code_%s = orc_program_take_code (p);\n",
